@@ -634,10 +634,10 @@ def cli(
                             forwards_df = pd.DataFrame(
                                 forwards_list,
                                 columns=[
-                                    "To",
-                                    "To_title",
-                                    "From",
-                                    "From_ID",
+                                    "Source",
+                                    "Target",
+                                    "Label",
+                                    "Source_ID",
                                     "Username",
                                     "Timestamp",
                                 ],
@@ -689,20 +689,20 @@ def cli(
                                                 forwards_df = pd.DataFrame(
                                                     forwards_list,
                                                     columns=[
-                                                        "To username",
-                                                        "To name",
-                                                        "From",
-                                                        "From ID",
-                                                        "From_username",
+                                                        "Source",
+                                                        "Target",
+                                                        "Label",
+                                                        "Source_ID",
+                                                        "Username",
                                                         "Timestamp",
                                                     ],
                                                 )
 
                                                 forwards_list.append(
                                                     [
+                                                        result,
                                                         t,
                                                         to_title,
-                                                        result,
                                                         f_from_id,
                                                         username,
                                                         timestamp,
@@ -894,18 +894,20 @@ def cli(
                                                         "Translated_text",
                                                         "Translation_confidence",
                                                         "Timestamp",
+                                                        "Has_media",
                                                         "Reply",
                                                         "Views",
+                                                        "URL",
                                                     ],
                                                 )
 
                                                 c_forwards = pd.DataFrame(
                                                     forwards_list,
                                                     columns=[
-                                                        "To",
-                                                        "To_title",
-                                                        "From",
-                                                        "From_ID",
+                                                        "Source",
+                                                        "Target",
+                                                        "Label",
+                                                        "Source_ID",
                                                         "Username",
                                                         "Timestamp",
                                                     ],
@@ -1045,7 +1047,6 @@ def cli(
                                                         #print("#### TODO: REACTIONS")
 
                                                 if media_archive == True:
-                                                    #add a progress bar for each file download
                                                     if message.media:
                                                         path = await message.download_media(
                                                             file=media_directory
@@ -1053,8 +1054,13 @@ def cli(
                                                         files.append(path)
                                                     else:
                                                         pass
-
                                                 
+                                                if message.media is not None:
+                                                    has_media = "TRUE"
+                                                else:
+                                                    has_media = 'FALSE'
+
+                                                post_url = "https://t.me/s/" + t + "/" + message.id
 
                                                 message_list.append(
                                                     [
@@ -1067,8 +1073,10 @@ def cli(
                                                         translated_text,
                                                         translation_confidence,
                                                         timestamp,
+                                                        has_media,
                                                         reply,
                                                         views,
+                                                        post_url,
                                                     ]
                                                 )
 
@@ -1158,17 +1166,15 @@ def cli(
 
                                                             forwards_list.append(
                                                                 [
+                                                                    result,
                                                                     t,
                                                                     to_title,
-                                                                    result,
                                                                     f_from_id,
                                                                     username,
                                                                     timestamp,
                                                                 ]
                                                             )
 
-                                                        
-
                                                     except ChannelPrivateError:
                                                         private_count += 1
                                                         continue
@@ -1191,6 +1197,11 @@ def cli(
                                                     "None",
                                                     "None",
                                                     "None",
+                                                    "None",
+                                                    "None",
+                                                    "None",
+                                                    "None",
+                                                    "None",
                                                 ]
                                             )
                                             pass
@@ -1199,7 +1210,6 @@ def cli(
                                         bar()
 
                                 if reply_analysis is True:
-                                    c_repliers_u = c_repliers["User id"].unique()
                                     if len(replies_list) > 0:
                                         with open(
                                             reply_file_archive, "w+", encoding="utf-8"
@@ -1210,7 +1220,7 @@ def cli(
                                         with open(
                                             reply_memberlist_filename, "w+", encoding="utf-8"
                                         ) as repliers_file:
-                                            c_repliers_u.to_csv(repliers_file, sep=";")
+                                            c_repliers.to_csv(repliers_file, sep=";")
 
                                 with open(
                                     file_archive, "w+", encoding="utf-8"
@@ -1307,8 +1317,6 @@ def cli(
                                     df04 = c_archive.Display_name.unique()
                                     plength = len(df03)
                                     unique_active = len(df04)
-                                    # one day this'll work out sleeping times
-                                    # print(c_t_stats)
 
                                 elif reply_analysis is True:
                                     if len(replies_list) > 0:
@@ -1443,12 +1451,11 @@ def cli(
                                         color_print_green(
                                             "  ├  Total unique repliers: ", str(replier_unique)
                                         )
-                                        # add a figure for unique current posters who are active
 
                                 if forwards_check is True:
                                     if forward_count >= 15:
                                         forwards_found = c_forwards.From.count()
-                                        value_count = c_forwards["From"].value_counts()
+                                        value_count = c_forwards["Source"].value_counts()
                                         c_f_stats = value_count.rename_axis(
                                             "unique_values"
                                         ).reset_index(name="counts")
@@ -1501,7 +1508,7 @@ def cli(
                                             + " forwarded messages"
                                         )
 
-                                        c_f_unique = c_forwards.From.unique()
+                                        c_f_unique = c_forwards.Source.unique()
                                         unique_forwards = len(c_f_unique)
 
                                         #print("\n")
@@ -1658,12 +1665,16 @@ def cli(
                             )
                         )
 
-                        # progress bar?
-
                         for user in result.updates[0].peers:
                             try:
                                 user_df = pd.DataFrame(
-                                    locations_list, columns=["User_ID", "Distance"]
+                                    locations_list, columns=[
+                                        "User_ID",
+                                        "Distance",
+                                        "Longitude",
+                                        "Latitude"
+                                        "Retrieval_time"
+                                        ]
                                 )
                                 if hasattr(user, "peer"):
                                     ID = user.peer.user_id
@@ -1674,7 +1685,7 @@ def cli(
                                 else:
                                     pass
 
-                                locations_list.append([ID, distance])
+                                locations_list.append([ID, distance, longitude, latitude, filetime])
 
                             except:
                                 pass
@@ -1701,8 +1712,8 @@ def cli(
                                 d_3000 += 1
 
                         with open(
-                            save_file, "w+", encoding="utf-8"
-                        ) as f:  # could one day append, including access time to differentiate
+                            save_file, "a+", encoding="utf-8"
+                        ) as f: 
                             user_df.to_csv(f, sep=";", index=False)
 
                         total = len(locations_list)
@@ -1714,9 +1725,6 @@ def cli(
                         color_print_green("  ├  Users within 3000m: ", str(d_3000))
                         color_print_green("  ├  Total users found:  ", str(total))
                         color_print_green("  └  Location list saved to: ", save_file)
-
-                        # can also do the same for channels with similar output file to users
-                        # may one day add trilateration to find users closest to exact point
                         
     with client:
         client.loop.run_until_complete(main())
